/* ©  Copyright Steve Simon, 2013 */
#include <u.h>
#include <libc.h>
#include <fcall.h>
#include <thread.h>
#include <9p.h>
#include <auth.h>
#include <bio.h>
#include <json.h>
#include "jsonfs.h"
#include "axosoft.h"

int
auth(Sess *s, char *pkgname)
{
	char *p;
	Jvalue *v;
	UserPasswd *user, *client;

	if((user = auth_getuserpasswd(auth_getkey, "proto=pass service=%s-user server=%s", pkgname, s->url)) == nil){
		werrstr("cannot get user credentials, %r");
		return -1;
	}

	if((client = auth_getuserpasswd(auth_getkey, "proto=pass service=%s-client server=%s", pkgname, s->url)) == nil){
		werrstr("cannot get client credentials, %r");
		return -1;
	}

	if((v = exch(s, nil, "api/oauth2/token?grant_type=password&username=%s&password=%s&client_id=%s&client_secret=%s&scope=read",
	  user->user, user->passwd, client->user, client->passwd)) == nil){
		werrstr("authorisation failed, %r");
		return -1;
	}

	memset(user->passwd, 0, strlen(user->passwd));
	free(user);

	memset(client->passwd, 0, strlen(client->passwd));
	free(client);


	if((p = json_strval(v, '.', "access_token")) == nil){
		werrstr("missing access token, %r");
		json_free(v);
		return -1;
	}

	s->headers = smprint("Authorization: Bearer %s", p);		/* BEWARE: Axosoft requires Bearer to be caplialised (sigh) */
	if(s->headers == nil){
		werrstr("no memory, %r");
		json_free(v);
		return -1;
	}

	if((p = json_strval(v, '.', "data.id")) == nil){
		werrstr("missing user id, %r");
		json_free(v);
		return -1;
	}
	s->userid = estrdup9p(p);

	json_free(v);
	return 0;
}
