#include <u.h>
#include <libc.h>
#include <fcall.h>
#include <thread.h>
#include <9p.h>
#include <bio.h>
#include <json.h>
#include "jsonfs.h"
#include "axosoft.h"

typedef struct Cache Cache;
struct Cache {
	char *src;
	char *dst;
	Cache *left;
	Cache *right;
};

static Cache *Users;

/* add to a binary tree (after K&R) */
static Cache *
add(Cache *c, char *src, char *dst)
{
	int n, ns, nd;

	if(c == nil){
		ns = strlen(src)+1;
		nd = strlen(dst)+1;
		c = emalloc9p(sizeof(Cache)+ns+nd);
		c->src = (char *)&c[1];
		c->dst = c->src +ns;
		strcpy(c->src, src);
		strcpy(c->dst, dst);
		return c;
	}

	n = strcmp(src, c->src);
	if(n < 0)
		c->left = add(c->left, src, dst);
	else
		c->right = add(c->right, src, dst);
	return c;
}

/* lookup a binary tree (after K&R) */
static char *
look(Cache *c, char *src)
{
	int n;

	if(c == nil)
		return nil;
	n = strcmp(src, c->src);
	if(n < 0)
		return look(c->left, src);
	if(n > 0)
		return look(c->right, src);
	return c->dst;
}


/* map an ID to a login name */
char *
id2login(Sess *s, char *id)
{
	char *p;
	Jvalue *v;
	static char *unk = "unknown";

	if((p = look(Users, id)) != nil)
		return p;

	if((v = exch(s, nil, "api/v5/users/%s", id)) == nil){
		Users = add(Users, id, unk);
		return unk;
	}
	if((p = json_strval(v, '.', "data.login_id")) == nil){
		json_free(v);
		Users = add(Users, id, unk);
	}
	Users = add(Users, id, p);
	free(v);
	return look(Users, id);
}

	
/* manually add to ID to login name map */
void
cacheuser(char *id, char *name)
{
	Users = add(Users, id, name);
}

