#include <u.h>
#include <libc.h>
#include <fcall.h>
#include <thread.h>
#include <auth.h>
#include <9p.h>
#include <bio.h>
#include <json.h>
#include "jsonfs.h"
#include "axosoft.h"

char *UriList = "columns=status,id";

/* Best efforts to give a short username */
static char *
username(Sess *s, Jvalue *v, char *base, char *name)
{
	char *p;

	if((p = json_strval(v, '.', "%s.%s.type", base, name)) != nil && *p)
		if(strcmp(p, "user") == 0){
			if((p = json_strval(v, '.', "%s.%s.id", base, name)) != nil && *p)
				return id2login(s, p);
		}
		else{
			if((p = json_strval(v, '.', "%s.%s.name", base, name)) != nil && *p)
				return p;
		}
	if((p = json_strval(v, '.', "%s.%s.id", base, name)) != nil && *p)
		return id2login(s, p);

	if((p = json_strval(v, '.', "%s.%s.name", base, name)) != nil && *p)
		return p;

	return "none";
}

/* add attachments */
static int
attachscan(Node *n, Sess *s)
{
	int i;
	Jvalue *v;
	long mtime;
	char *p, *url, *uid, *item_id;

	item_id = n->aux;
	if((v = exch(s, nil, "api/v5/%s/attachments", item_id)) == nil){
		warn("cannot scan for attachments, %r\n");
		return -1;
	}

	for(i = 0; (p = json_strval(v, '.', "data[%d].created_by.id", i)) != nil; i++){
		uid = id2login(s, p);
		mtime = 0;
		if((p = json_strval(v, '.', "data[%d].attach_date", i)) != nil && *p)
			mtime = iso2sec(p);
		if((p = json_strval(v, '.', "data[%d].id", i)) != nil && *p){
			url = smprint("api/v5/attachments/%s/data", p);
			if((p = json_strval(v, '.', "data[%d].file_name", i)) != nil && *p)
				newurl(n, p, uid, uid, uid, mtime, mtime, url);
			free(url);
		}
		
	}
	json_free(v);
	return 0;
}

/* add emails */
static int
emailscan(Node *n, Sess *s)
{
	int i;
	Node *ch;
	Jvalue *v;
	char *item_id;

	item_id = n->aux;
	if((v = exch(s, nil, "api/v5/%s/emails", item_id)) == nil){
		warn("cannot scan for emails, %r\n");
		return -1;
	}

	SET(ch, i);
	USED(ch, i);
/*
	for(i = 0; (p = json_strval(v, '.', "data[%d].id", i)) != nil; i++){
		ch = newdir(n, nil, nil, nil, p);
		ch->aux = estrdup9p(p);
		ch->scan = addattach;
	}
*/
	json_free(v);
	return 0;
}


/* add notifications */
static int
notifyscan(Node *n, Sess *s)
{
	int i;
	Node *ch;
	Jvalue *v;
	char *item_id;

	item_id = n->aux;
	if((v = exch(s, nil, "api/v5/%s/notifications", item_id)) == nil){
		warn("cannot scan for notifications, %r\n");
		return -1;
	}

	SET(ch, i);
	USED(ch, i);
/*
	for(i = 0; (p = json_strval(v, '.', "data[%d].id", i)) != nil; i++){
		ch = newdir(n, nil, nil, nil, p);
		ch->aux = estrdup9p(p);
		ch->scan = addattach;
	}
*/
	json_free(v);
	return 0;
}



/* add comments */
static int
commentscan(Node *n, Sess *s)
{
	int i;
	Node *ch;
	Jvalue *v;
	char *item_id;

	item_id = n->aux;
	if((v = exch(s, nil, "api/v5/%s/comments", item_id)) == nil){
		warn("cannot scan for comments, %r\n");
		return -1;
	}

	SET(ch, i);
	USED(ch, i);
/*
	for(i = 0; (p = json_strval(v, '.', "data[%d].id", i)) != nil; i++){
		ch = newdir(n, nil, nil, nil, p);
		ch->aux = estrdup9p(p);
		ch->scan = addattach;
	}
*/
	json_free(v);
	return 0;
}


/* add an item (feature/task/incident/defect) */
static int
add(Node *n, Sess *s, char *item, char *prefix, Jvalue *v)
{
	Fmt f;
	Node *ch;
	char *p, *uid;
	ulong mtime, atime;

	uid = 0;
	if((p = json_strval(v, '.', "%s.assigned_to.id", prefix)) != nil)
		uid = id2login(s, p);

	mtime = atime = 0;
	if((p = json_strval(v, '.', "%s.last_updated_date_time", prefix)) != nil  && *p){
		mtime = atime = iso2sec(p);
		newfile(n, "upddated", uid, uid, uid, mtime, atime, ctime(mtime));
	}

	if((p = json_strval(v, '.', "%s.reported_date.name", prefix)) != nil && *p)
		newfile(n, "reported", uid, uid, uid, mtime, atime, ctime(iso2sec(p)));
	if((p = json_strval(v, '.', "%s.description", prefix)) != nil && *p)
		newfile(n, "description", uid, uid, uid, mtime, atime, p);
	if((p = json_strval(v, '.', "%s.notes", prefix)) != nil && *p)
		newfile(n, "notes", uid, uid, uid, mtime, atime, p);
	if((p = json_strval(v, '.', "%s.severity", prefix)) != nil && *p)
		newfile(n, "severity", uid, uid, uid, mtime, atime, p);
	if((p = json_strval(v, '.', "%s.created_date_time", prefix)) != nil && *p)
		newfile(n, "created", uid, uid, uid, mtime, atime, ctime(iso2sec(p)));
	if((p = json_strval(v, '.', "%s.reported_date.name", prefix)) != nil && *p)
		newfile(n, "reported", uid, uid, uid, mtime, atime, ctime(iso2sec(p)));
	if((p = json_strval(v, '.', "%s.project.name", prefix)) != nil && *p)
		newfile(n, "project", uid, uid, uid, mtime, atime, p);
	if((p = json_strval(v, '.', "%s.name", prefix)) != nil && *p)
		newfile(n, "name", uid, uid, uid, mtime, atime, p);
	if((p = json_strval(v, '.', "%s.replication_procedures", prefix)) != nil && *p)
		newfile(n, "reproduce", uid, uid, uid, mtime, atime, p);
	if((p = json_strval(v, '.', "%s.resolution", prefix)) != nil && *p)
		newfile(n, "resolution", uid, uid, uid, mtime, atime, p);

	newfile(n, "creator", uid, uid, uid, mtime, atime, username(s, v, prefix, "created_by"));
	newfile(n, "reporter", uid, uid, uid, mtime, atime, username(s, v, prefix, "reported_by"));
	newfile(n, "assignee", uid, uid, uid, mtime, atime, username(s, v, prefix, "assigned_to"));
	newfile(n, "updater", uid, uid, uid, mtime, atime, username(s, v, prefix, "last_updated_by"));

	if((p = json_strval(v, '.', "%s.reported_by_customer_contact.name")) != nil && *p){
		fmtstrinit(&f);
		fmtprint(&f, "%s\n", p);
		if((p = json_strval(v, '.', "%s.reported_by_customer_contact.email", prefix)) != nil && *p)
			fmtprint(&f, "email: %s\n", p);
		if((p = json_strval(v, '.', "%s.reported_by_customer_contact.phone", prefix)) != nil && *p)
			fmtprint(&f, "phone: %s\n", p);
		p = fmtstrflush(&f);
		newfile(n, "customer", uid, uid, uid, mtime, atime, p);
		free(p);
	}

	if((p = json_strval(v, '.', "%s.project.name", prefix)) != nil && *p)
		newfile(n, "project", uid, uid, uid, mtime, atime, p);
 	if((p = json_strval(v, '.', "%s.release", prefix)) != nil && *p)
		newfile(n, "release", uid, uid, uid, mtime, atime, p);
 
	if((p = json_strval(v, '.', "%s.status.name", prefix)) != nil && *p)
		newfile(n, "status", uid, uid, uid, mtime, atime, p);
	else
	if((p = json_strval(v, '.', "%s.status.id", prefix)) != nil && *p)
		newfile(n, "status", uid, uid, uid, mtime, atime, p);
 
	if((p = json_strval(v, '.', "%s.workflow_step.name", prefix)) != nil && *p)
		newfile(n, "workflow", uid, uid, uid, mtime, atime, p);
 	else
	if((p = json_strval(v, '.', "%s.workflow_step.id", prefix)) != nil && *p)
		newfile(n, "workflow", uid, uid, uid, mtime, atime, p);

  	if((p = json_strval(v, '.', "%s.priority.name", prefix)) != nil && *p)
		newfile(n, "priority", uid, uid, uid, mtime, atime, p);
  	else
	if((p = json_strval(v, '.', "%s.priority.id", prefix)) != nil && *p)
		newfile(n, "priority", uid, uid, uid, mtime, atime, p);


	if((p = json_strval(v, '.', "%s.id", prefix)) != nil && *p){
		ch = newdir(n, uid, uid, uid, "attachments");
		ch->scan = attachscan;
		ch->aux = smprint("%s/%s", item, p);

   		ch = newdir(n, uid, uid, uid, "comments");
		ch->scan = commentscan;
		ch->aux = smprint("%s/%s", item, p);

 		ch = newdir(n, uid, uid, uid, "emails");
		ch->scan = emailscan;
		ch->aux = smprint("%s/%s", item, p);
	
 		ch = newdir(n, uid, uid, uid, "notifications");
		ch->scan = notifyscan;
		ch->aux = smprint("%s/%s", item, p);
	}
	return 0;
}

/* This style is slower (~350ms/call), but works - but does return status, priotrity, and workflow names */
int
additem(Node *n, Sess *s, char *item)
{
	int rc;
	char *id;
	Jvalue *v;

	id = n->aux;
	if((v = exch(s, nil, "api/v5/%s?search_field=id&search_string=%s", item, id)) == nil){		/* FIXME: awaiting search_field syntax for arrays */
		warn("cannot scan for one %s, %r\n", item);
		return -1;
	}
	rc = add(n, s, item, "data[0]", v);
	json_free(v);
	return rc;
}

/* This style is faster (~100ms/call), but broken - it doesn't return status, priotrity, or workflow names */
int
x_additem(Node *n, Sess *s, char *item)
{
	int rc;
	char *id;
	Jvalue *v;

	id = n->aux;
	if((v = exch(s, nil, "api/v5/%s/%s", item, id)) == nil){
		warn("cannot scan for one %s, %r\n", item);
		return -1;
	}
	rc = add(n, s, item, "data", v);
	json_free(v);
	return rc;
}


/* This style is faster, if you want them all, but slower if you want only one... */
int
additems(Node *n, Sess *s, char *item)
{
	int i;
	Node *ch;
	Jvalue *v;
	char prefix[32], *p;

	if((v = exch(s, nil, "api/v5/%s&%s", item, UriList)) == nil){
		warn("cannot scan for all %s, %r\n", item);
		return -1;
	}

	for(i = 0; (p = json_strval(v, '.', "data[%d].id", i)) != nil; i++){
		ch = newdir(n, nil, nil, nil, p);
		ch->aux = estrdup9p(p);
		snprint(prefix, sizeof(prefix), "data[%d]", i);
		add(ch, s, item, prefix, v);
	}

	json_free(v);
	return 0;
}

