#include <u.h>
#include <libc.h>
#include <fcall.h>
#include <thread.h>
#include <auth.h>
#include <9p.h>
#include <bio.h>
#include <json.h>
#include "jsonfs.h"
#include "axosoft.h"

/* request a users open detects */
static int
openbyuser(Node *n, Sess *s)
{
	int i;
	Node *ch;
	Jvalue *v;
	char *status, *uid, *id;

	uid = n->aux;
	if((v = exch(s, nil, "api/v5/defects?assigned_to_id=%s&%s", uid, UriList)) == nil){	
		warn("cannot scan for user's defects, %r\n");
		return -1;
	}

	for(i = 0; (id = json_strval(v, '.', "data[%d].id", i)) != nil; i++){
		if((status = json_strval(v, '.', "data[%d].status.name", i)) == nil)
			continue;
		if(cistrcmp(status, "open") != 0)
			continue;

		ch = newdir(n, nil, nil, nil, id);
		ch->aux = estrdup9p(id);
		ch->scan = adddefect;
	}
	json_free(v);
	return 0;
}

/* request a users fixed (ready for test) detects */
static int
fixedbyuser(Node *n, Sess *s)
{
	int i;
	Node *ch;
	Jvalue *v;
	char *status, *uid, *id;

	uid = n->aux;
	if((v = exch(s, nil, "api/v5/defects?assigned_to_id=%s&%s", uid, UriList)) == nil){	
		warn("cannot scan for user's defects, %r\n");
		return -1;
	}

	for(i = 0; (id = json_strval(v, '.', "data[%d].id", i)) != nil; i++){
		if((status = json_strval(v, '.', "data[%d].status.name", i)) == nil)
			continue;
		if(cistrcmp(status, "fixed") != 0)
			continue;

		ch = newdir(n, nil, nil, nil, id);
		ch->aux = estrdup9p(id);
		ch->scan = adddefect;
	}
	json_free(v);
	return 0;
}

/* request a users closed (signed off) detects */
static int
closedbyuser(Node *n, Sess *s)
{
	int i;
	Node *ch;
	Jvalue *v;
	char *status, *uid, *id;

	uid = n->aux;
	if((v = exch(s, nil, "api/v5/defects?assigned_to_id=%s&%s", uid, UriList)) == nil){	
		warn("cannot scan for user's defects, %r\n");
		return -1;
	}

	for(i = 0; (id = json_strval(v, '.', "data[%d].id", i)) != nil; i++){
		if((status = json_strval(v, '.', "data[%d].status.name", i)) == nil)
			continue;
		if(cistrcmp(status, "closed") != 0)
			continue;

		ch = newdir(n, nil, nil, nil, id);
		ch->aux = estrdup9p(id);
		ch->scan = adddefect;
	}
	json_free(v);
	return 0;
}

/* request a users fixed (ready for test) detects */
static int
allbyuser(Node *n, Sess *s)
{
	int i;
	Node *ch;
	Jvalue *v;
	char *p, *uid;

	uid = n->aux;
	if((v = exch(s, nil, "api/v5/defects?assigned_to_id=%s&%s", uid, UriList)) == nil){	
		warn("cannot scan for user's defects, %r\n");
		return -1;
	}

	for(i = 0; (p = json_strval(v, '.', "data[%d].id", i)) != nil; i++){
		ch = newdir(n, nil, nil, nil, p);
		ch->aux = estrdup9p(p);
		ch->scan = adddefect;
	}
	json_free(v);
	return 0;
}

static int
featuresbyuser(Node *n, Sess *s)
{
	int i;
	Node *ch;
	Jvalue *v;
	char *p, *id;

	id = n->aux;
	if((v = exch(s, nil, "api/v5/features?%s&assigned_to_id=%s", UriList, id)) == nil){
		warn("cannot scan for user's defects, %r\n");
		return -1;
	}

	for(i = 0; (p = json_strval(v, '.', "data[%d].id", i)) != nil; i++){
		ch = newdir(n, nil, nil, nil, p);
		ch->aux = estrdup9p(p);
		ch->scan = addfeature;
	}
	json_free(v);
	return 0;
}

static int
tasksbyuser(Node *n, Sess *s)
{
	int i;
	Node *ch;
	Jvalue *v;
	char *p, *id;

	id = n->parent->aux;
	if((v = exch(s, nil, "api/v5/tasks?%s&assigned_to_id=%s", UriList, id)) == nil){
		warn("cannot scan for user's tasks, %r\n");
		return -1;
	}

	for(i = 0; (p = json_strval(v, '.', "data[%d].id", i)) != nil; i++){
		ch = newdir(n, nil, nil, nil, p);
		ch->aux = estrdup9p(p);
		ch->scan = addtask;
	}
	json_free(v);
	return 0;
}


static int
incidentsbyuser(Node *n, Sess *s)
{
	int i;
	Node *ch;
	Jvalue *v;
	char *p, *id;

	id = n->aux;
	if((v = exch(s, nil, "api/v5/incidents?%s&assigned_to_id=%s", UriList, id)) == nil){
		warn("cannot scan for user's incidents, %r\n");
		return -1;
	}

	for(i = 0; (p = json_strval(v, '.', "data[%d].id", i)) != nil; i++){
		ch = newdir(n, nil, nil, nil, p);
		ch->aux = estrdup9p(p);
		ch->scan = addincident;
	}
	json_free(v);
	return 0;
}

static int
defectsbyuser(Node *n, Sess *)
{
	Node *ch;

	ch = newdir(n, nil, nil, nil, "open");
	ch->scan = openbyuser;
	ch->last = Permenant;
	ch->aux = estrdup9p(n->aux);

	ch = newdir(n, nil, nil, nil, "fixed");
	ch->scan = fixedbyuser;
	ch->last = Permenant;
	ch->aux = estrdup9p(n->aux);

	ch = newdir(n, nil, nil, nil, "closed");
	ch->scan = closedbyuser;
	ch->last = Permenant;
	ch->aux = estrdup9p(n->aux);

	ch = newdir(n, nil, nil, nil, "all");
	ch->scan = allbyuser;
	ch->last = Permenant;
	ch->aux = estrdup9p(n->aux);
	return 0;
}


/* add a single user */
static void
adduser(Node *n, Jvalue *v, char *prefix, char *id)
{
	Fmt f;
	int i;
	Node *ch;
	char *p, *uid;
	long mtime, atime, created;

	n->aux = estrdup9p(id);

	mtime = 0;
	atime = 0;
	created = 0;
	if((p = json_strval(v, '.', "%s.created_date_time", prefix)) != nil && *p)
		created = iso2sec(p);
	if((p = json_strval(v, '.', "%s.last_updated_date_time", prefix)) != nil && *p)
		mtime = iso2sec(p);
	if((p = json_strval(v, '.', "%s.last_login_date_time", prefix)) != nil && *p)
		atime = iso2sec(p);
	uid = json_strval(v, '.', "%s.login_id", prefix);

	newfile(n, "id", uid, uid, uid, mtime, atime, id);
	newfile(n, "created", uid, uid, uid, mtime, atime, ctime(created));
	newfile(n, "updated", uid, uid, uid, mtime, atime, ctime(mtime));
	newfile(n, "lastlogin", uid, uid, uid, mtime, atime, ctime(atime));
	
	if((p = json_strval(v, '.', "%s.login_id", prefix)) != nil && *p)
		newfile(n, "login", uid, uid, uid, mtime, atime, p);
	if((p = json_strval(v, '.', "%s.full_name", prefix)) != nil && *p)
		newfile(n, "fullname", uid, uid, uid, mtime, atime, p);
	if((p = json_strval(v, '.', "%s.windows_id", prefix)) != nil && *p)
		newfile(n, "windows_id", uid, uid, uid, mtime, atime, p);
	if((p = json_strval(v, '.', "%s.email", prefix)) != nil && *p)
		newfile(n, "email", uid, uid, uid, mtime, atime, p);
	if((p = json_strval(v, '.', "%s.is_active", prefix)) != nil && *p)
		newfile(n, "active", uid, uid, uid, mtime, atime, p);
	if((p = json_strval(v, '.', "%s.is_locked", prefix)) != nil && *p)
		newfile(n, "locked", uid, uid, uid, mtime, atime, p);
	if((p = json_strval(v, '.', "%s.is_active", prefix)) != nil && *p)
		newfile(n, "active", uid, uid, uid, mtime, atime, p);
	if((p = json_strval(v, '.', "%s.is_admin", prefix)) != nil && *p)
		newfile(n, "administrator", uid, uid, uid, 0, 0, p);
	if((p = json_strval(v, '.', "%s.culture_info", prefix)) != nil && *p)
		newfile(n, "culture", uid, uid, uid, mtime, atime, p);
	if((p = json_strval(v, '.', "%s.failed_logins", prefix)) != nil && *p)
		newfile(n, "failedlogins", uid, uid, uid, 0, 0, p);

	fmtstrinit(&f);
	for(i = 0; (p = json_strval(v, '.', "%s.security_roles[%d].name", prefix, i)) != nil && *p; i++)
		fmtprint(&f, "%s\n", p);
	p = fmtstrflush(&f);
	newfile(n, "roles", uid, uid, uid, mtime, atime, p);
	free(p);

	ch = newdir(n, uid, uid, uid, "features");
	ch->scan = featuresbyuser;
	ch->aux = estrdup9p(id);

	ch = newdir(n, uid, uid, uid, "tasks");
	ch->scan = tasksbyuser;
	ch->aux = estrdup9p(id);

	ch = newdir(n, uid, uid, uid, "incidents");
	ch->scan = incidentsbyuser;
	ch->aux = estrdup9p(id);

	ch = newdir(n, uid, uid, uid, "defects");
	ch->scan = defectsbyuser;
	ch->aux = estrdup9p(id);

	cacheuser(id, uid);
}

/* get all users, add them, one per directory */
int
alluserscan(Node *n, Sess *s)
{
	int i;
	Node *ch;
	Jvalue *v;
	char *id, *uid, buf[32];

	if((v = exch(s, nil, "api/v5/users")) == nil){
		warn("cannot scan for users, %r\n");
		return -1;
	}

	for(i = 0; (uid = json_strval(v, '.', "data[%d].login_id", i)) != nil && *uid; i++){
		ch = newdir(n, uid, uid, uid, uid);
		if((id = json_strval(v, '.', "data[%d].id", i)) != nil && *id){
			snprint(buf, sizeof(buf), "data[%d]", i);
			adduser(ch, v, buf, id);
		}
	}
	json_free(v);
	return 0;
}

/* get one user and add them to this directory */
int
oneuserscan(Node *n, Sess *s)
{
	char *id;
	Jvalue *v;

	id = n->aux;
	if((v = exch(s, nil, "api/v5/users/%s", id)) == nil){
		warn("cannot scan for my user id=%s, %r\n", id);
		return -1;
	}
	adduser(n, v, "data", s->userid);
	json_free(v);
	return 0;
}

/* get my user info (login user) and add it to this directory */
int
localscan(Node *n, Sess *s)
{
	Jvalue *v;

	if((v = exch(s, nil, "api/v5/users/%s", s->userid)) == nil){
		warn("cannot scan for userid, %r\n");
		return -1;
	}
	adduser(n, v, "data", s->userid);
	json_free(v);
	return 0;
}

